///////////////////////////////////////////////////////////
//  ofxPAR_ControlManager.cpp
//  Implementation of the Class ofxPAR_ControlManager
//  Created on:      14-fev-2015 00:12:00
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////

#include "ofxPAR_ControlManager.h"

#include "ofxPAR_C_Button.h"
#include "ofxPAR_C_Slider.h"
#include "ofxPAR_C_Switch.h"

using cv::Mat;
using cv::RotatedRect;

const ofColor ofxPAR_ControlManager::accent1 = ofColor(100, 0, 57);
const ofColor ofxPAR_ControlManager::accent2 = ofColor(45, 0, 180);

ofxPAR_ControlManager::ofxPAR_ControlManager(){
	doControlDetection = true;

    BORDER = 8;
    ALPHA = 0.7;
}



ofxPAR_ControlManager::~ofxPAR_ControlManager(){

}




void ofxPAR_ControlManager::setup(){
	
	finder.setMinAreaRadius(20);
	finder.setMaxAreaRadius(120);
    // Don't threshold, will run edge detection instead
    finder.setAutoThreshold(false);

	controlList.clear();
}

//---------------------------------------------------------
void ofxPAR_ControlManager::reset() {

	controlList.clear();

}

//---------------------------------------------------------
void ofxPAR_ControlManager::detect(Mat img) {
    // Get the image in the right format and run edge detection - cheat - tricks
    ofxCv::convertColor(img, grayImg, CV_RGB2GRAY); //converte a imagem para cinza
    cv::Canny(grayImg, edgesInput, 160, 180, 3); // extrapola as bordas - cria edge map
	canny = edgesInput;
    cv::dilate(edgesInput, edgesInput, Mat::ones(2, 2, CV_8U), cv::Point(-1, -1), 7); //dilata as bordas
	dilated = edgesInput;
    cv::erode(edgesInput, edgesInput, Mat::ones(2, 2, CV_8U), cv::Point(-1, -1), 5);  // erode as bordas
	eroded = edgesInput;

    const cv::Rect &roi = cv::Rect(BORDER, BORDER, edgesInput.cols - 2*BORDER, edgesInput.rows - 2*BORDER);
    edgesInput(roi).copyTo(edges);

    // Find contours in the edge detected image
    finder.findContours(edges);

    size_t n = finder.size();

	// Check to find the type of control:
    for (size_t i = 0; i < n; i++) {
        if (isButton(i)) {
            float radius;
            cv::Point center = finder.getMinEnclosingCircle(i, radius);
			ofxPAR_C_Button *b = new ofxPAR_C_Button(center.x, center.y, radius);

			controlList.push_back(b);
        } else if (isSlider(i)) {
            cv::RotatedRect rect = finder.getMinAreaRect(i);
            ofxPAR_C_Slider *s = new ofxPAR_C_Slider(rect);
			controlList.push_back(s);
        } else if (isSwitch(i)) {
            cv::RotatedRect rect = finder.getMinAreaRect(i);
            ofxPAR_C_Switch *s = new ofxPAR_C_Switch(rect);
			controlList.push_back(s);
        }
    }
    
    assignControls();


}


//---------------------------------------------------------
void ofxPAR_ControlManager::assignControls() {

	vector<ofxPAR_Control*>::iterator itC;
	int i = 0;

	for( itC = controlList.begin() ; itC != controlList.end() ; itC++){
		(*itC)->setId(i);
		(*itC)->setColor(ofColor::green);
		i++;
	}
}


//---------------------------------------------------------
void ofxPAR_ControlManager::processInteraction(const ofPoint &point) {
    lastInputPoint = point;

	vector<ofxPAR_Control*>::iterator itC;

	for(itC = controlList.begin() ; itC != controlList.end() ; itC++)
		if ( (*itC)->onInteraction(point)) break;
            // Stop after the first control to handle this input
}

//---------------------------------------------------------
void ofxPAR_ControlManager::drawControls() {

	vector<ofxPAR_Control*>::iterator itC;
	for(itC = controlList.begin() ; itC != controlList.end() ; itC++)
		(*itC)->draw();

    ofSetColor(accent1);
    ofFill();
    ofCircle(lastInputPoint.x, lastInputPoint.y, 5);
}

void ofxPAR_ControlManager::drawDetectorInput(float x, float y, float w, float h) {
    ofxCv::drawMat(edges, x, y, w, h);
}

void ofxPAR_ControlManager::drawDebug(){


}


//---------------------------------------------------------
bool ofxPAR_ControlManager::isButton(size_t i) {
    float radius;
    finder.getMinEnclosingCircle(i, radius);
    float circleArea = PI * radius * radius;
    double contourArea = finder.getContourArea(i);
    return abs(1.0 - circleArea / contourArea) < 0.3;
}

//---------------------------------------------------------
bool ofxPAR_ControlManager::isSlider(size_t i) {
    float radius;
    ofVec2f center = ofxCv::toOf(finder.getMinEnclosingCircle(i, radius));
    float circleArea = PI * radius * radius;
    double contourArea = finder.getContourArea(i);
    return circleArea / contourArea > 12;
}

//---------------------------------------------------------
bool ofxPAR_ControlManager::isSwitch(size_t i) {
    RotatedRect rr = finder.getMinAreaRect(i);
    float rectArea = rr.size.width * rr.size.height;
    double contourArea = finder.getContourArea(i);
    return abs(1.0 - rectArea / contourArea) < 0.25
        && abs(1.0 - (float) rr.size.width / rr.size.height) > 0.1;
}
