///////////////////////////////////////////////////////////
//  ofxPAR_Infra.cpp
//  Implementation of the Class ofxPAR_Infra
//  Created on:      14-fev-2015 00:11:55
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////
#include "ofxXmlSettings.h"

#include "ShapeUtils.h"

#include "ofxPAR_Infra.h"

using namespace ofxCv;
using namespace cv;

ofxPAR_Infra::ofxPAR_Infra(){
	
	// SETUP INFRA FOUND FLAG
	foundInfra = false;

	// SETUP CAMERA 
	//camWidth = 1024;
	//camHeight = 768;
	camWidth = 640;
	camHeight = 480;

	// SETUP CAMERA
	cam.initGrabber(camWidth,camHeight);
	cam.listDevices();

	this->showVideoFeed = false;

	// ALLOCATE RAW IMAGE
	unwarped.allocate(camWidth, camHeight, OF_IMAGE_COLOR); // 

	// SETUP 
	this->projWidth = ofGetWindowWidth();
	this->projHeight = ofGetWindowHeight();


}

void ofxPAR_Infra::toogleVideoFeed(){
	
	if(this->showVideoFeed == true) {
		showVideoFeed = false;
	} else {
		showVideoFeed = true;
	}
}

ofxPAR_Infra::~ofxPAR_Infra(){

}


//bool ofxPAR_Infra::setState( AppState new_state){
//
//	vector<ofxPAR_Infra_State*>::iterator itIS;
//
//	for( itIS = stateList.begin() ; itIS != stateList.end(); itIS++){
//		if( (*itIS)->stateType == new_state){
//			this->actualState = (*itIS);
//			ofLogNotice() << "New State: "<< this->actualState->name;
//			return true;
//			}
//	}
//	return false;
//}




//---------------------------------------------------------
//     PROJECTOR-CAMERA CALIBRATION METHODS - STEP 1
//---------------------------------------------------------

void ofxPAR_Infra::doCalibration(){
	this->computeProjectorAlignment();
	if(!this->saveProjectorAlignment()){
            ofLog(OF_LOG_WARNING, "Could not save alignment.xml, projector alignment will be lost on exit.");
	}
}

//---------------------------------------------------------
void ofxPAR_Infra::addCalibrationPoints(int x, int y) { 
	if(projectorPoints.size() < 4) { 
		this->projectorPoints.push_back(Point2f(x,y));
	}
}

//---------------------------------------------------------
void ofxPAR_Infra::resetProjectorAlignment() {
	projectorPoints.clear();
	toProjectorMatrix = Mat::eye(3, 3, CV_32F);
	alignmentComplete = false;
}

//---------------------------------------------------------
bool ofxPAR_Infra::saveProjectorAlignment() {
	if (projectorPoints.size() != 4) {
		return false;
	}

	ofxXmlSettings alignment;
	alignment.addTag("alignment");
	alignment.pushTag("alignment");
	for (size_t i = 0; i < projectorPoints.size(); i++) {
		alignment.addTag("position");
		alignment.pushTag("position", i);
		alignment.addValue("x", projectorPoints[i].x);
		alignment.addValue("y", projectorPoints[i].y);
		alignment.popTag();
	}
	alignment.popTag();

	// TODO If ever upgraded beyond oF 0.700, this returns a boolean
	alignment.saveFile("alignment.xml");
	return true;
}

//---------------------------------------------------------
bool ofxPAR_Infra::loadProjectorAlignment() { //look on an alignment.xml for earlier projection alignment.
	ofxXmlSettings alignment;
	if (!alignment.loadFile("alignment.xml")) { //try to find file
		return false; }

	alignment.pushTag("alignment"); 
	if (alignment.getNumTags("position") != 4) {
		return false;   }

	projectorPoints.clear();
	for (size_t i = 0; i < 4; i++) {
		alignment.pushTag("position", i);

		Point2f p;
		p.x = alignment.getValue("x", 0);
		p.y = alignment.getValue("y", 0);
		projectorPoints.push_back(p);

		alignment.popTag();
	}
	alignment.popTag();

	computeProjectorAlignment();

	return true;
}

//---------------------------------------------------------
void ofxPAR_Infra::computeProjectorAlignment() {
	vector<Point2f> dstPoints(4); // Limitantes da projeo

	dstPoints[0] = Point2f(0, 0); 
	dstPoints[1] = Point2f(ofGetWindowWidth(), 0);
	dstPoints[2] = Point2f(ofGetWindowWidth(), ofGetWindowHeight());
	dstPoints[3] = Point2f(0, ofGetWindowHeight());

	// projectorPoints  do tipo:  vector<cv::Point2f>
	// toProjectorMatrix  do tipo: cv::Mat - de dimenso 3x3
	toProjectorMatrix = getPerspectiveTransform(&projectorPoints[0], &dstPoints[0]); //cria matrix de perspectiva
	alignmentComplete = true;
}


//---------------------------------------------------------
void ofxPAR_Infra::drawProjectionLimits(){
	// Draw the projection rectangle
	// This rectangle is used to sync cam-proj
	ofNoFill();
	ofSetLineWidth(5);
	ofSetColor(ofColor::white);
	ofRect(0, 0, ofGetWindowWidth(), ofGetWindowHeight());

	ofFill();
	ofSetColor(ofColor::green);
	ofCircle(0,0,40);
	ofCircle(ofGetWindowWidth(),0,40);
	ofCircle(0,ofGetWindowHeight(),40);
	ofCircle(ofGetWindowWidth(),ofGetWindowHeight(),40);

}
