///////////////////////////////////////////////////////////
//  ofxPAR_Infra_PaperDetector.cpp
//  Implementation of the Class ofxPAR_Infra_PaperDetector
//  Created on:      14-fev-2015 00:11:56
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////
#include "ofxXmlSettings.h"

#include "ShapeUtils.h"
#include "ofxPAR_Infra_PaperDetector.h"




using namespace ofxCv;
using namespace cv;


ofxPAR_Infra_PaperDetector::ofxPAR_Infra_PaperDetector() : ofxPAR_Infra() {

	this->name = "[INFRA][PAPERDETECTOR]";


	//LOAD PROJECTOR CALIBRATION
	if (!loadProjectorAlignment()) {
		ofLog(OF_LOG_NOTICE, "No saved alignment found, starting from scratch.");
		resetProjectorAlignment();
	}

	this->configureCV();

}



ofxPAR_Infra_PaperDetector::~ofxPAR_Infra_PaperDetector(){

}

void ofxPAR_Infra_PaperDetector::draw(){

//	this->actualState->draw();

	this->drawPaperContour();

}

void ofxPAR_Infra_PaperDetector::update(){

	// warp backgroud
    if (this->foundInfra) {
		unwarp(unwarped);
        unwarped.update();
	}



}

//---------------------------------------------------------
//     PAPER TRACKER METHODS - STEP 2
//---------------------------------------------------------

//---------------------------------------------------------
void ofxPAR_Infra_PaperDetector::configureCV(){
	ofLogNotice("[PAPER DETECTOR]") << " Configure CV ";

	contourFinder.setMinAreaRadius(10); // confiugra menor rea .
	contourFinder.setMaxAreaRadius(150); // configura maior rea
	targetColor = ofColor::white; // inicializa a cor sendo buscada
	trackingColorMode = TRACK_COLOR_RGB; //inicializa o tipo padro de cor rastreada
	thresholdCV = 100; // inicializa o nvel da camera.
	contourFinder.setTargetColor(targetColor, trackingColorMode); // configura cor / padro
	contourFinder.setThreshold(thresholdCV); // configura nvel

	

}

//---------------------------------------------------------
void ofxPAR_Infra_PaperDetector::detect() {

	contourFinder.findContours(cam); // realiza  a procura por folhas brancas

   vector<cv::Point> maxQuad;
   float maxArea = -numeric_limits<float>::infinity();

	int n = contourFinder.size();
	//ofLogNotice() << "[PAPER DETECTOR] " + ofToString(n);

	for(int i = 0; i < n; i++) {
		contour = toOf(contourFinder.getMinAreaRect(i)); 		// encontra o menor retangulo do contorno e retorna as linhas laterais
		vector<cv::Point> quad = contourFinder.getFitQuad(i);   // encontra o menor retangulo e retorna as quinas
		//contour = toOf(quad);

        float area = ShapeUtils::polylineArea(contour);
        if (area > maxArea) {
            maxArea = area;
            maxQuad = quad;
        }

	}

    // Make sure it's a rectangle
    bool isRect = ShapeUtils::isRectangle(maxQuad);
    if (isRect) {
        paperImage = ofxCv::toCv(cam);
		paper = maxQuad;

		adaptedToScreenSize.clear();
		adaptedToScreenSize = paper;
    }

	this->foundInfra = isRect;
}



void ofxPAR_Infra_PaperDetector::drawPaperContourIn(int w, int h){

	if(this->foundInfra){ //Only draw after found something.
		ofSetColor(ofColor::green);
		ofxCv::toOf(this->adaptedToScreenSize).draw();
	}

}

//---------------------------------------------------------
void ofxPAR_Infra_PaperDetector::drawPaperContour(){

	if(this->foundInfra){
		//Only draw after found something.
		ofNoFill();
		ofSetColor(ofColor::red);
		ofxCv::toOf(paper).draw();
	}

}


void ofxPAR_Infra_PaperDetector::setup(){

	this->configureCV();
}


Mat ofxPAR_Infra_PaperDetector::getTransformation(int outWidth, int outHeight) {
    vector<Point2f> warpPoints;
    copy(paper.begin(), paper.end(), back_inserter(warpPoints));
    ShapeUtils::orderQuadForTransform(warpPoints);

    vector<Point2f> dstPoints(4);
    dstPoints[0] = Point2f(0, 0);
    dstPoints[1] = Point2f(outWidth, 0);
    dstPoints[2] = Point2f(outWidth, outHeight);
    dstPoints[3] = Point2f(0, outHeight);

    return cv::getPerspectiveTransform(&dstPoints[0], &warpPoints[0]);
}



ofPoint ofxPAR_Infra_PaperDetector::unwarpPointIntern(const ofPoint &point, int outWidth, int outHeight) {
    vector<Point2f> warpPoints;
    copy(paper.begin(), paper.end(), back_inserter(warpPoints));
    ShapeUtils::orderQuadForTransform(warpPoints);

    vector<Point2f> dstPoints(4);
    dstPoints[0] = Point2f(0, 0);
    dstPoints[1] = Point2f(outWidth, 0);
    dstPoints[2] = Point2f(outWidth, outHeight);
    dstPoints[3] = Point2f(0, outHeight);

    Mat transform = cv::getPerspectiveTransform(&warpPoints[0], &dstPoints[0]);
    float pv[2] = {point.x, point.y};
    Mat srcPoint = Mat(1, 1, CV_32FC2, pv);
    Mat dstPoint;
    cv::perspectiveTransform(srcPoint, dstPoint, transform);

    dstPoint = dstPoint.reshape(1);
    return ofPoint(dstPoint.at<float>(0), dstPoint.at<float>(1));
}


