///////////////////////////////////////////////////////////
//  ofxPAR_State_Idle.cpp
//  Implementation of the Class ofxPAR_State_Idle
//  Created on:      19-fev-2015 10:30:54
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////

#include "ofxPAR_State_Idle.h"

#include "ofxProjAR.h"
#include "ofxPAR_Infra.h"

#include "ofxPAR_ControlManager.h"


#include "ShapeUtils.h"

using namespace ofxCv;
using namespace cv;

ofxPAR_State_Idle::ofxPAR_State_Idle(){
	this->name = "[OFXPAR][STATE][IDLE]";
	this->stateType = AppState::IDLE;
}



ofxPAR_State_Idle::~ofxPAR_State_Idle(){

}





void ofxPAR_State_Idle::draw(){
	
	// In the Idle view, we capture a debug view of everything:

	//frame 0,0
	ofSetColor(ofColor::white);
	this->app->m_infra->cam.draw(10,10,320,240);

	// Draw paper and hand overlays on camera
    ofPushMatrix();
    ofTranslate(10,10);
    ofScale(0.5, 0.5);
	app->m_actuator->draw();
    ofPopMatrix();

	ofNoFill();
	ofSetLineWidth(1);
	ofRect(10,10,320,240);


	//frame 0,1
	    // Draw processed background subtraction
	drawMat(this->app->m_actuator->foreground, 340, 10, 320, 240);

	ofNoFill();
	ofSetLineWidth(1);
	ofRect(340, 10, 320, 240);

	//frame 0,2
    // Draw processed background subtraction
	this->app->m_actuator->drawDetectorInput(690, 10, 320, 240);

	ofNoFill();
	ofSetLineWidth(1);
	ofRect(690, 10, 320, 240);

	//frame 1,0 - canny

	
	if( app->m_infra->isInfraFound()) this->app->m_infra->getRawUnwarped().draw(10,260,320,240);

	ofNoFill();
	ofSetColor(ofColor::white);
	ofSetLineWidth(1);
	ofRect(10, 260, 320, 240);

	//frame 1,1 - canny
	if( app->m_infra->isInfraFound()) ofxCv::drawMat(this->app->controlManager.canny,340,260,320,240);


	ofNoFill();
	ofSetLineWidth(1);
	ofRect(340,260,320,240);

	//frame 1,2 - dilated
	if( app->m_infra->isInfraFound()) ofxCv::drawMat(this->app->controlManager.dilated,690,260,320,240);


	ofNoFill();
	ofSetLineWidth(1);
	ofRect(690,260,320,240);

	//frame 1,3 - eroded
	if( app->m_infra->isInfraFound()) ofxCv::drawMat(this->app->controlManager.eroded,1030,260,320,240);


	ofNoFill();
	ofSetLineWidth(1);
	ofRect(1030,260,320,240);

	//frame 2,0
	if( app->m_infra->isInfraFound()) this->app->controlManager.drawDetectorInput(10, 510, 320, 240);

	ofNoFill();
	ofSetLineWidth(1);
	ofRect(10, 510, 320, 240);


}


void ofxPAR_State_Idle::update(){
	
	if(this->app->m_infra->cam.isFrameNew()){

		// Look up for infra:
		this->app->m_infra->detect();

		// Do the infra job 
		// in the case of paper --> warp the sheet area;
		this->app->m_infra->update();

		// Do the control manager job
		// Find the physical controls
		// The virtual only controls doesnt net to be detected
        if (this->app->controlManager.doControlDetection) {
            this->app->controlManager.detect(this->app->m_infra->getRawUnwarped());
            this->app->controlManager.doControlDetection = false;
        }

		//Create new camera instance picture to transfer to actuator detector
		
		Mat paperCamMat = toCv(this->app->m_infra->cam);
        Mat paperCamChan = Mat::zeros(this->app->m_infra->cam.height, this->app->m_infra->cam.width, CV_8UC3);
        int fromTo[] = { 1,0 , 1,1 , 1,2 };
        mixChannels(&paperCamMat, 1, &paperCamChan, 1, fromTo, 3);

		app->m_actuator->topBackground.update(paperCamChan, app->m_actuator->foreground);
		if (app->m_actuator->detect(app->m_actuator->foreground, app->m_infra->getContours())) {
			ofPoint rawPoint = app->m_actuator->getActuatorPoint();
          //  app->controlManager.processInteraction(app->m_infra->unwarpPoint(rawPoint, this->app->m_infra->unwarped.width, this->app->m_infra->unwarped.height));
        }

	}


}