///////////////////////////////////////////////////////////
//  ofxPAR_State_InfraSetup.cpp
//  Implementation of the Class ofxPAR_State_InfraSetup
//  Created on:      19-fev-2015 11:02:37
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////

#include "ofxPAR_State_InfraSetup.h"

#include "ofxProjAR.h"
#include "ofxPAR_Infra.h"

#include "ShapeUtils.h"


using namespace ofxCv;
using namespace cv;


ofxPAR_State_InfraSetup::ofxPAR_State_InfraSetup(){
		this->name = "[OFXPAR][STATE][SETUPINFRASTRUCTURE]";
		this->stateType = AppState::CONFIGINFRA;
}



ofxPAR_State_InfraSetup::~ofxPAR_State_InfraSetup(){

}





void ofxPAR_State_InfraSetup::draw(){
	// In this state we are gonna show in two phase: 
	// 1st - with the alignment points - second with the aligment projection

	// Show camera feed
	ofSetColor(ofColor::white);


	if(this->app->m_infra->showVideoFeed) {
		this->app->m_infra->cam.draw(0,0);
		this->app->m_infra->drawProjectionLimits();
	}

	
	if( !this->app->m_infra->alignmentComplete){
		// Draw the projection rectangle. This rectangle is used to sync cam-proj
				
		ofFill(); // to show where are the projection points to calibration
		ofSetColor(0, 255, 0);

		for (size_t i = 0; i < this->app->m_infra->projectorPoints.size(); i++) {
			Point2f &pt = this->app->m_infra->projectorPoints[i];
			ofCircle(pt.x, pt.y, 5);
		}
	} else {
		// Calibration is complete, testing perspective
		ofFill();
		ofSetColor(ofColor::white);
		ofRect(0,0,ofGetWidth(), ofGetHeight());

		ofPushMatrix();
			ofSetColor(34, 183, 220);
			ShapeUtils::applyTransform(this->app->m_infra->toProjectorMatrix);
			ofPolyline quad = ofxCv::toOf(this->app->m_infra->projectorPoints);
			ofPoint c = ShapeUtils::getCentroid2D(quad);
			ofFill();
			ofCircle(c.x, c.y, 20);
			ofCircle(c.x + 50, c.y, 20);
			ofCircle(c.x, c.y + 50, 20);
			ofCircle(c.x - 50, c.y, 20);
			ofCircle(c.x, c.y - 50, 20);

			this->app->m_infra->draw();

		ofPopMatrix();
	}

	// Draw performance statistics
	ofSetColor(ofColor::pink);
	// draw found infra info:
	if (this->app->m_infra->isInfraFound()) { ofDrawBitmapString("Paper detected", 10, ofGetHeight() - 25 - 10);
    } else { ofDrawBitmapString("No paper", 10, ofGetHeight() - 25 - 10); }
	// draw alingment info:
	if (this->app->m_infra->alignmentComplete) { ofDrawBitmapString("Alignment Complete - Press 'r' to reset alignment", 10, ofGetHeight() - 25 - 30);
	} else { ofDrawBitmapString("Alignment incomplete - number of points:" + ofToString(this->app->m_infra->projectorPoints.size()), 10, ofGetHeight() - 25 - 30); }

}


void ofxPAR_State_InfraSetup::update(){
	// as this is a camera based interaction always check if the frame is new to drop fps 
	// the app might as well run into about 100fps, and your camera runs into about 30fps
	if(this->app->m_infra->cam.isFrameNew()){
		// In this state we are gonna calibrate proj-cam and find infra

		// Calibration

		if (!this->app->m_infra->alignmentComplete && this->app->m_infra->projectorPoints.size() == 4 ) {
			this->app->m_infra->doCalibration();
		}	

		// Look for infra (if cam-proj alignment is complete)
		if( this->app->m_infra->alignmentComplete) this->app->m_infra->detect();
	}

}