///////////////////////////////////////////////////////////
//  ofxProjAR.cpp
//  Implementation of the Class ofxProjAR
//  Created on:      14-fev-2015 00:11:54
//  Original author: Christopher Shneider Cerqueira
///////////////////////////////////////////////////////////

#include "ofxXmlSettings.h"

#include "ofxProjAR.h"
#include "ShapeUtils.h"

#include "ofxPAR_State_Idle.h"
#include "ofxPAR_State_InfraSetup.h"
#include "ofxPAR_State_Play.h"

#include "ofxPAR_Infra_PaperDetector.h"

#include "ofxPAR_HandDetector.h"


using namespace ofxCv;
using namespace cv;


ofxProjAR::ofxProjAR(){


	//allocate idle
	ofxPAR_State_Idle *id = new ofxPAR_State_Idle();
	id->app = this;
	this->stateList.push_back(id);

	//allocate setup infra;
	ofxPAR_State_InfraSetup *si = new ofxPAR_State_InfraSetup();
	si->app = this;
	this->stateList.push_back(si);

	////allocate play
	ofxPAR_State_Play *pl = new ofxPAR_State_Play();
	pl->app = this;
	this->stateList.push_back(pl);


	////set initial active state: idle
	this->actualState = id;

	debug = false;
}



ofxProjAR::~ofxProjAR(){

}


void ofxProjAR::standardSetup(){
	ofEnableSmoothing(); //
    ofBackground(0); //

	// SETUP INFRA
	this->m_infra = new ofxPAR_Infra_PaperDetector();

	// SETUP CONTROL MANAGER
	controlManager.setup();
	
	
	// SETUP ACTUATOR
	this->m_actuator = new ofxPAR_HandDetector();

	
	// SETUP INITIAL STATE;
	this->setState(AppState::IDLE);

	
}





void ofxProjAR::standardUpdate(){
	// update camera;
	this->m_infra->cam.update();

	this->actualState->update();

}

void ofxProjAR::standardDraw(){
	this->actualState->draw();

}



bool ofxProjAR::setState( AppState new_state){

	vector<ofxPAR_State*>::iterator itIS;

	for( itIS = stateList.begin() ; itIS != stateList.end(); itIS++){
		if( (*itIS)->stateType == new_state){
			this->actualState = (*itIS);
			ofLogNotice() << "New State: "<< this->actualState->name;
			return true;
			}
	}
	return false;
}
